#!/bin/bash

. /etc/default/${EPAGES_PROJECT:-epages6}
PERL_EXE=$EPAGES_PERL/bin/perl

ERROR=$(ls Makefile.PL patch.pl $PERL_EXE 2>&1 >/dev/null)
RESULT=$?
if [[ $RESULT != 0 ]] ; then
  echo "patch.sh: $ERROR"
  exit $RESULT
fi

hostname=$(hostname)
PATCH_TAG=$(awk -F= '/^PATCH_TAG[[:space:]]*=/{gsub(/[[:space:]]+/,""); print $2}' Makefile.PL)
LOG_DIR=${EPAGES_PATCHES:-"$EPAGES_SHARED/Patches"}/var-$PATCH_TAG/logs/$hostname
mkdir -m 755 -p $LOG_DIR || exit 1

#------------------------------------------------------------------------------
# change perl
#------------------------------------------------------------------------------

unset PERL_HAS_CHANGED

grep -q 'EPAGES_CARTRIDGES=$2' $PERL_EXE || [[ ! -f sh/6017005000/EPAGES_PERL/bin/perl ]] || {

  if [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] ; then
    # if not running on debian but on redhat/suse
    # then test if function epages_perl5lib from /var/epages/functions.sh exists
    if ! epages_perl5lib >/dev/null ; then
      # if not
      if EPAGES_RELEASE_6_17_5=$(rpm -q --whatprovides 'epages-release(6.17.5)') ; then
        # either epages-release is up-to-date, but corrupt
        echo "Package epages-release is corrupt, exiting."
      else
        # or epages-release is outdated
        echo $EPAGES_RELEASE_6_17_5
        echo "Package epages-release is outdated, exiting."
        echo "First update epages-release by executing:"
        echo "  rpm -hUv http://epages.com/repo/epages/yum/epages-release.noarch.rpm"
      fi
      exit 1
    fi
  fi

  # $EPAGES_PERL/bin/perl is outdated and new is available
  cat sh/6017005000/EPAGES_PERL/bin/perl > $PERL_EXE || exit 1
}

# do not update perl if perl flex server is running
service ePagesConfigServer status 2>/dev/null && SKIP_PERL_UPDATE=1

# update perl if libssl.so is outdated and new is available
LIBSSL=lib32/ssl/libssl.so
NEW_LIBSSL=$(ls -v sh/6017*/EPAGES_PERL/$LIBSSL 2>/dev/null | tail -1)
grep -q '1\.0\.0n' $EPAGES_PERL/$LIBSSL || [[ ! -f $NEW_LIBSSL ]] || {
  PERL_HAS_CHANGED=
  for i in $(ls -d ${NEW_LIBSSL%/*}/*.so* 2>/dev/null) ; do
    # update library here
    [[ -z $SKIP_PERL_UPDATE ]] && cat $i > $EPAGES_PERL/${i##*/EPAGES_PERL/}
    PERL_HAS_CHANGED=1
  done
  for i in $(grep -l ${LIBSSL%/*}/ sh/*/patch.conf 2>/dev/null) ; do
    # therefore delete library entries in sh/*/patch.conf
    sed -i "s,^${LIBSSL%/*}/.*,," $i
    # sh/*/patch.conf has changed -> delete entry in filelist.txt
    if [[ -n $PERL_HAS_CHANGED && -f filelist.txt ]] ; then
      cp -p filelist.txt filelist.orig
      grep -v $i filelist.orig > filelist.txt
    fi
  done
  # and remember this
  [[ -z $SKIP_PERL_UPDATE && -n $PERL_HAS_CHANGED ]] && touch ${LOG_DIR%/*}/PERL_HAS_CHANGED
}

if [[ -f ${LOG_DIR%/*}/PERL_HAS_CHANGED && ! -f $LOG_DIR/PERL_WAS_RESTARTED ]] ; then
  # due to changed binaries we must restart the application server
  if [[ -x ${EPSERVICE:-"$EPAGES/Service/epagesService"} ]] ; then
    EPAGES_IGNORE=cron /etc/init.d/${EPAGES_PROJECT:-epages6} start_service
  fi
  touch $LOG_DIR/PERL_WAS_RESTARTED
fi

#------------------------------------------------------------------------------
# run perl
#------------------------------------------------------------------------------

date=$(date +'%Y-%h-%d-%H.%M.%S')

rm -f /tmp/epages-patch-result.log

$EPAGES_PERL/bin/perl patch.pl "$@" 2>&1 | tee $LOG_DIR/$date-${1////-}.log

# return success or failure

RESULT=${PIPESTATUS[0]}
if [[ -s /tmp/epages-patch-result.log ]] ; then
  PATCH_RESULT=$(tail -1 /tmp/epages-patch-result.log)
  [[ -n $PATCH_RESULT && -z ${PATCH_RESULT//[0-9]/} ]] && RESULT=$PATCH_RESULT
fi

exit $RESULT
